@@ -1,15 +1,17 @@
+from typing import Dict, List, Optional, Set, TYPE_CHECKING, Tuple, Union
 from collections import defaultdict
 from inspect import signature
+from copy import deepcopy
 import json
 import os
 import time
-from typing import Dict, List, Optional, Set, TYPE_CHECKING, Tuple
 import uuid
 
 import ray
 from ray import ObjectRef, logger
 from ray.actor import ActorClass
 from ray.tune.resources import Resources
+from ray.util.annotations import PublicAPI, DeveloperAPI
 from ray.util.placement_group import PlacementGroup, get_placement_group, \
     placement_group, placement_group_table, remove_placement_group
 
@@ -47,6 +49,7 @@ def get_tune_pg_prefix():
     return _tune_pg_prefix
 
 
+@PublicAPI(stability="beta")
 class PlacementGroupFactory:
     """Wrapper class that creates placement groups for trials.
 
@@ -88,6 +91,21 @@ class PlacementGroupFactory:
     could be used e.g. if you had one learner running in the main trainable
     that schedules two remote workers that need access to 2 CPUs each.
 
+    If the trainable itself doesn't require resources.
+    You can specify it as:
+
+    .. code-block:: python
+
+        from ray import tune
+
+        tune.run(
+            train,
+            resources_per_trial=tune.PlacementGroupFactory([
+                {},
+                {"CPU": 2},
+                {"CPU": 2},
+            ], strategy="PACK"))
+
     Args:
         bundles(List[Dict]): A list of bundles which
             represent the resources requirements.
@@ -104,12 +122,22 @@ class PlacementGroupFactory:
     """
 
     def __init__(self,
-                 bundles: List[Dict[str, float]],
+                 bundles: List[Dict[str, Union[int, float]]],
                  strategy: str = "PACK",
                  *args,
                  **kwargs):
+        assert len(bundles) > 0, (
+            "Cannot initialize a PlacementGroupFactory with zero bundles.")
+        if not bundles[0]:
+            # This is when trainable itself doesn't need resources.
+            self._head_bundle_is_empty = True
+            bundles.pop(0)
+        else:
+            self._head_bundle_is_empty = False
+
         self._bundles = [{k: float(v)
-                          for k, v in bundle.items()} for bundle in bundles]
+                          for k, v in bundle.items() if v != 0}
+                         for bundle in bundles]
         self._strategy = strategy
         self._args = args
         self._kwargs = kwargs
@@ -120,8 +148,25 @@ class PlacementGroupFactory:
         self._bind()
 
     @property
-    def head_cpus(self):
-        return self._bundles[0].get("CPU", None)
+    def head_bundle_is_empty(self):
+        """Returns True if head bundle is empty while child bundles
+        need resources.
+
+        This is considered an internal API within Tune.
+        """
+        return self._head_bundle_is_empty
+
+    @property
+    @DeveloperAPI
+    def head_cpus(self) -> float:
+        return 0.0 if self._head_bundle_is_empty else self._bundles[0].get(
+            "CPU", 0.0)
+
+    @property
+    @DeveloperAPI
+    def bundles(self) -> List[Dict[str, float]]:
+        """Returns a deep copy of resource bundles"""
+        return deepcopy(self._bundles)
 
     @property
     def required_resources(self) -> Dict[str, float]:
@@ -428,29 +473,31 @@ class PlacementGroupManager:
         self._in_use_pgs[pg] = trial
         self._in_use_trials[trial] = pg
 
-        # We still have to pass resource specs
-        # Pass the full resource specs of the first bundle per default
-        first_bundle = pg.bundle_specs[0].copy()
-        num_cpus = first_bundle.pop("CPU", None)
-        num_gpus = first_bundle.pop("GPU", None)
-
-        # Only custom resources remain in `first_bundle`
-        resources = first_bundle or None
-
-        if num_cpus is None:
-            # If the placement group specifically set the number
-            # of CPUs to 0, use this.
-            num_cpus = pgf.head_cpus
-
         logger.debug(f"For trial {trial} use pg {pg.id}")
 
-        return actor_cls.options(
-            placement_group=pg,
-            placement_group_bundle_index=0,
-            placement_group_capture_child_tasks=True,
-            num_cpus=num_cpus,
-            num_gpus=num_gpus,
-            resources=resources)
+        # We still have to pass resource specs
+        if not pgf.head_bundle_is_empty:
+            # Pass the full resource specs of the first bundle per default
+            head_bundle = pg.bundle_specs[0].copy()
+            num_cpus = head_bundle.pop("CPU", 0)
+            num_gpus = head_bundle.pop("GPU", 0)
+
+            # Only custom resources remain in `head_bundle`
+            resources = head_bundle
+            return actor_cls.options(
+                placement_group=pg,
+                placement_group_bundle_index=0,
+                placement_group_capture_child_tasks=True,
+                num_cpus=num_cpus,
+                num_gpus=num_gpus,
+                resources=resources)
+        else:
+            return actor_cls.options(
+                placement_group=pg,
+                placement_group_capture_child_tasks=True,
+                num_cpus=0,
+                num_gpus=0,
+                resources={})
 
     def has_ready(self, trial: "Trial", update: bool = False) -> bool:
         """Return True if placement group for trial is ready.
@@ -495,9 +542,6 @@ class PlacementGroupManager:
                 no placement group was replaced.
 
         """
-        if not trial.uses_placement_groups:
-            return None
-
         pgf = trial.placement_group_factory
 
         staged_pg = self._unstage_unused_pg(pgf)
@@ -578,11 +622,7 @@ class PlacementGroupManager:
         Returns:
             Boolean indicating if the placement group was returned.
         """
-        if not trial.uses_placement_groups:
-            return True
-
         pgf = trial.placement_group_factory
-
         pg = self._in_use_trials.pop(trial)
         self._in_use_pgs.pop(pg)
 
@@ -648,34 +688,6 @@ class PlacementGroupManager:
 
         return trial_pg
 
-    def clean_trial_placement_group(
-            self, trial: "Trial") -> Optional[PlacementGroup]:
-        """Remove reference to placement groups associated with a trial.
-
-        Returns an associated placement group. If the trial was scheduled, this
-        is the placement group it was scheduled on. If the trial was not
-        scheduled, it will first try to return a staging placement group. If
-        there is no staging placement group, it will return a ready placement
-        group that is not yet being used by another trial.
-
-        Args:
-            trial ("Trial"): :obj:`Trial` object.
-
-        Returns:
-            PlacementGroup or None.
-
-        """
-        pgf = trial.placement_group_factory
-
-        if trial in self._in_use_trials:
-            # "Trial" was in use. Just return its placement group.
-            trial_pg = self._in_use_trials.pop(trial)
-            self._in_use_pgs.pop(trial_pg)
-        else:
-            trial_pg = self._unstage_unused_pg(pgf)
-
-        return trial_pg
-
     def in_staging_grace_period(self):
         return self._staging_futures and self._grace_period and time.time(
         ) <= self._latest_staging_start_time + self._grace_period
@@ -701,9 +713,6 @@ class PlacementGroupManager:
         # Count number of expected placement groups
         pgf_expected: Dict[PlacementGroupFactory, int] = defaultdict(int)
         for trial in trials:
-            if not trial.uses_placement_groups:
-                continue
-
             # Count in-use placement groups
             if trial in self._in_use_trials:
                 current_counts[trial.placement_group_factory] += 1
@@ -753,37 +762,3 @@ class PlacementGroupManager:
                     resources[key] = resources.get(key, 0) + val
 
         return resources
-
-    def total_used_resources(self, committed_resources: Resources) -> dict:
-        """Dict of total used resources incl. placement groups
-
-        Args:
-            committed_resources (Resources): Additional commited resources
-                from (legacy) Ray Tune resource management.
-        """
-        committed = committed_resources._asdict()
-
-        # Make dict compatible with pg resource dict
-        committed.pop("has_placement_group", None)
-        committed["CPU"] = committed.pop("cpu", 0) + committed.pop(
-            "extra_cpu", 0)
-        committed["GPU"] = committed.pop("gpu", 0) + committed.pop(
-            "extra_gpu", 0)
-        committed["memory"] += committed.pop("extra_memory", 0.)
-        committed["object_store_memory"] += committed.pop(
-            "extra_object_store_memory", 0.)
-
-        custom = committed.pop("custom_resources", {})
-        extra_custom = committed.pop("extra_custom_resources", {})
-
-        for k, v in extra_custom.items():
-            custom[k] = custom.get(k, 0.) + v
-
-        committed.update(custom)
-
-        pg_resources = self.occupied_resources()
-
-        for k, v in committed.items():
-            pg_resources[k] = pg_resources.get(k, 0.) + v
-
-        return pg_resources
